@@ -1,7 +1,7 @@
 /*
  *  db.c
  * 
- *  Copyright (c) 2002-2005 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -327,12 +327,6 @@ pkginfo_t* db_read(pacdb_t *db, struct dirent *ent, unsigned int inforeq)
 					return(NULL);
 				}
 				trim(info->url);
-			} else if(!strcmp(line, "%LICENSE%")) {
-				if(fgets(info->license, sizeof(info->license), fp) == NULL) {
-					FREEPKG(info);
-					return(NULL);
-				}
-				trim(info->license);
 			} else if(!strcmp(line, "%ARCH%")) {
 				if(fgets(info->arch, sizeof(info->arch), fp) == NULL) {
 					FREEPKG(info);
@@ -386,6 +380,11 @@ pkginfo_t* db_read(pacdb_t *db, struct dirent *ent, unsigned int inforeq)
 				}
 				trim(tmp);
 				info->size = atol(tmp);
+			} else if(!strcmp(line, "%LICENSE%")) {
+				while(fgets(line, 512, fp) && strlen(trim(line))) {
+					char *s = strdup(line);
+					info->license = list_add(info->license, s);
+				}
 			} else if(!strcmp(line, "%REPLACES%")) {
 				/* the REPLACES tag is special -- it only appears in sync repositories,
 				 * not the local one.  */
@@ -529,7 +528,10 @@ int db_write(pacdb_t *db, pkginfo_t *info, unsigned int inforeq)
 		fputs("%URL%\n", fp);
 		fprintf(fp, "%s\n\n", info->url);
 		fputs("%LICENSE%\n", fp);
-		fprintf(fp, "%s\n\n", info->license);
+		for(lp = info->license; lp; lp = lp->next) {
+			fprintf(fp, "%s\n", (char*)lp->data);
+		}
+		fprintf(fp, "\n");
 		fputs("%ARCH%\n", fp);
 		fprintf(fp, "%s\n\n", info->arch);
 		fputs("%BUILDDATE%\n", fp);
@@ -649,15 +651,13 @@ void db_search(pacdb_t *db, PMList *cache, const char *treename, PMList *needles
 
 	for(i = needles; i; i = i->next) {
 		char *targ = strdup(i->data);
-		strtoupper(targ);
 		for(j = cache; j; j = j->next) {
 			pkginfo_t *pkg = (pkginfo_t*)j->data;
 			char *haystack;
 			int match = 0;
 			/* check name */
 			haystack = strdup(pkg->name);
-			strtoupper(haystack);
-			if(strstr(haystack, targ)) {
+			if(reg_match(haystack, targ)) {
 				match = 1;
 			}
 			FREE(haystack);
@@ -665,8 +665,7 @@ void db_search(pacdb_t *db, PMList *cache, const char *treename, PMList *needles
 			/* check description */
 			if(!match) {
 				haystack = strdup(pkg->desc);
-				strtoupper(haystack);
-				if(strstr(haystack, targ)) {
+				if(reg_match(haystack, targ)) {
 					match = 1;
 				}
 				FREE(haystack);
@@ -679,8 +678,7 @@ void db_search(pacdb_t *db, PMList *cache, const char *treename, PMList *needles
 				if(info != NULL) {
 					for(m = info->provides; m; m = m->next) {
 						haystack = strdup(m->data);
-						strtoupper(haystack);
-						if(strstr(haystack, targ)) {
+						if(reg_match(haystack, targ)) {
 							match = 1;
 						}
 						FREE(haystack);
@@ -762,71 +760,92 @@ PMList* db_find_conflicts(pacdb_t *db, PMList *targets, char *root, PMList **ski
 		pkginfo_t *p = (pkginfo_t*)i->data;
 		pkginfo_t *dbpkg = NULL;
 		for(j = p->files; j; j = j->next) {
+			int isdir = 0;
 			filestr = (char*)j->data;
 			snprintf(path, PATH_MAX, "%s%s", root, filestr);
-			if(!stat(path, &buf) && !S_ISDIR(buf.st_mode)) {
+			/* is this target a file or directory? */
+			if(path[strlen(path)-1] == '/') {
+				isdir = 1;
+				path[strlen(path)-1] = '\0';
+			}
+			if(!lstat(path, &buf)) {
 				int ok = 0;
-				if(dbpkg == NULL) {
-					dbpkg = db_scan(db, p->name, INFRQ_DESC | INFRQ_FILES);
-
-					if(dbpkg)
-						strhash_add_list(htables[target_num], dbpkg->files);
+				if(!S_ISLNK(buf.st_mode) && ((isdir && !S_ISDIR(buf.st_mode)) || (!isdir && S_ISDIR(buf.st_mode)))) {
+					/* if the package target is a directory, and the filesystem target
+					 * is not (or vice versa) then it's a conflict
+					 */
+					ok = 0;
+					goto donecheck;
 				}
-				if(dbpkg && strhash_isin(htables[target_num], filestr)) {
+				/* re-fetch with stat() instead of lstat() */
+				stat(path, &buf);
+				if(S_ISDIR(buf.st_mode)) {
+					/* if it's a directory, then we have no conflict */
 					ok = 1;
-				}
-				/* Make sure that the supposedly-conflicting file is not actually just
-				 * a symlink that points to a path that used to exist in the package.
-				 */
-				/* Check if any part of the conflicting file's path is a symlink */
-				if(dbpkg && !ok) {
-					MALLOC(str, PATH_MAX);
-					for(k = dbpkg->files; k; k = k->next) {
-						snprintf(str, PATH_MAX, "%s%s", root, (char*)k->data);
-						stat(str, &buf2);
-						if(buf.st_ino == buf2.st_ino && buf.st_dev == buf2.st_dev) {
-							printf("inodes match: %s and %s\n", path, str);
-							ok = 1;
-						}
+				} else {
+					if(dbpkg == NULL) {
+						dbpkg = db_scan(db, p->name, INFRQ_DESC | INFRQ_FILES);
+
+						if(dbpkg)
+							strhash_add_list(htables[target_num], dbpkg->files);
 					}
-					FREE(str);
-				}
-				/* Check if the conflicting file has been moved to another package/target */
-				if(!ok) {
-					/* Look at all the targets */
-					for(k = targets; k && !ok; k = k->next) {
-						pkginfo_t *p1 = (pkginfo_t*)k->data;
-						/* As long as they're not the current package */
-						if(strcmp(p1->name, p->name)) {
-							pkginfo_t *dbpkg2 = NULL;
-							dbpkg2 = db_scan(db, p1->name, INFRQ_DESC | INFRQ_FILES);
-							/* If it used to exist in there, but doesn't anymore */
-							if(dbpkg2 && !is_in(filestr, p1->files) && is_in(filestr, dbpkg2->files)) {
-								/*printf("file %s moved from %s to %s\n", filestr, p1->name, p->name);*/
-								
+					if(dbpkg && strhash_isin(htables[target_num], filestr)) {
+						ok = 1;
+					}
+					/* Make sure that the supposedly-conflicting file is not actually just
+					 * a symlink that points to a path that used to exist in the package.
+					 */
+					/* Check if any part of the conflicting file's path is a symlink */
+					if(dbpkg && !ok) {
+						MALLOC(str, PATH_MAX);
+						for(k = dbpkg->files; k; k = k->next) {
+							snprintf(str, PATH_MAX, "%s%s", root, (char*)k->data);
+							stat(str, &buf2);
+							if(buf.st_ino == buf2.st_ino && buf.st_dev == buf2.st_dev) {
+								/*printf("inodes match: %s and %s\n", path, str);*/
 								ok = 1;
-								/* Add to the "skip list" of files that we shouldn't remove during an upgrade.
-								 *
-								 * This is a workaround for the following scenario:
-								 *
-								 *    - the old package A provides file X
-								 *    - the new package A does not
-								 *    - the new package B provides file X
-								 *    - package A depends on B, so B is upgraded first
-								 *
-								 * Package B is upgraded, so file X is installed.  Then package A
-								 * is upgraded, and it *removes* file X, since it no longer exists
-								 * in package A.
-								 *
-								 * Our workaround is to scan through all "old" packages and all "new"
-								 * ones, looking for files that jump to different packages.
-								 */
-								*skip_list = list_add(*skip_list, filestr);
 							}
-							FREEPKG(dbpkg2);
+						}
+						FREE(str);
+					}
+					/* Check if the conflicting file has been moved to another package/target */
+					if(!ok) {
+						/* Look at all the targets */
+						for(k = targets; k && !ok; k = k->next) {
+							pkginfo_t *p1 = (pkginfo_t*)k->data;
+							/* As long as they're not the current package */
+							if(strcmp(p1->name, p->name)) {
+								pkginfo_t *dbpkg2 = NULL;
+								dbpkg2 = db_scan(db, p1->name, INFRQ_DESC | INFRQ_FILES);
+								/* If it used to exist in there, but doesn't anymore */
+								if(dbpkg2 && !is_in(filestr, p1->files) && is_in(filestr, dbpkg2->files)) {
+									/*printf("file %s moved from %s to %s\n", filestr, p1->name, p->name);*/
+								
+									ok = 1;
+									/* Add to the "skip list" of files that we shouldn't remove during an upgrade.
+									 *
+									 * This is a workaround for the following scenario:
+									 *
+									 *    - the old package A provides file X
+									 *    - the new package A does not
+									 *    - the new package B provides file X
+									 *    - package A depends on B, so B is upgraded first
+									 *
+									 * Package B is upgraded, so file X is installed.  Then package A
+									 * is upgraded, and it *removes* file X, since it no longer exists
+									 * in package A.
+									 *
+									 * Our workaround is to scan through all "old" packages and all "new"
+									 * ones, looking for files that jump to different packages.
+									 */
+									*skip_list = list_add(*skip_list, filestr);
+								}
+								FREEPKG(dbpkg2);
+							}
 						}
 					}
 				}
+donecheck:
 				if(!ok) {
 					MALLOC(str, 512);
 					snprintf(str, 512, "%s: %s: exists in filesystem", p->name, path);
