@@ -1,7 +1,7 @@
 /*
  *  pacman.h
  * 
- *  Copyright (c) 2002-2005 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@
 #define min(X, Y)  ((X) < (Y) ? (X) : (Y))
 
 int pacman_add(pacdb_t *db, PMList *targets, PMList *dependonly);
-int pacman_remove(pacdb_t *db, PMList *targets);
+int pacman_remove(pacdb_t *db, PMList *targets, PMList *skiplist);
 int pacman_upgrade(pacdb_t *db, PMList *targets, PMList *dependonly);
 int pacman_query(pacdb_t *db, PMList *targets);
 int pacman_sync(pacdb_t *db, PMList *targets);
