@@ -1,7 +1,7 @@
 /*
  *  pacsync.c
  * 
- *  Copyright (c) 2002-2005 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -52,6 +52,7 @@ extern char *pmo_xfercommand;
 
 extern unsigned short pmo_proxyport;
 extern unsigned short pmo_nopassiveftp;
+extern unsigned short pmo_noprogressbar;
 extern unsigned short pmo_chomp;
 
 /* sync servers */
@@ -441,6 +442,13 @@ int downloadfiles_forreal(PMList *servers, const char *localpath,
 					snprintf(completefile, PATH_MAX, "%s/%s", localpath, fn);
 					rename(output, completefile);
 				} else if(filedone < 0) {
+					if(!pmo_xfercommand) {
+						if(!strcmp(server->protocol, "ftp") && !pmo_proxyhost) {
+							 FtpQuit(control);
+						} else if(!strcmp(server->protocol, "http") || (pmo_proxyhost && strcmp(server->protocol, "file"))) {
+							 HttpQuit(control);
+						}
+					}
 					return(-1);
 				}
 				printf("\n");
@@ -476,6 +484,10 @@ static int log_progress(netbuf *ctl, int xfered, void *arg)
 	static unsigned short mouth;
 	static unsigned int   lastcur = 0;
 
+	if(pmo_noprogressbar) {
+		return(1);
+	}
+
 	gettimeofday(&t1, NULL);
 	if(xfered+offset == fsz) {
 		t = t0;
