@@ -1,7 +1,7 @@
 /*
  *  util.h
  * 
- *  Copyright (c) 2002-2005 by Judd Vinet <jvinet@zeroflux.org>
+ *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  * 
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -34,10 +34,12 @@ int unpack(char *archive, const char *prefix, const char *fn);
 int copyfile(char *src, char *dest);
 int makepath(char *path);
 int rmrf(char *path);
+char *rel2abs(const char *path, char *result, const size_t size);
 void indentprint(char *str, int indent);
 char* trim(char *str);
 char* strtoupper(char *str);
 int grep(const char *fn, const char *needle);
+int reg_match(char *string, char *pattern);
 
 #endif
 /* vim: set ts=2 sw=2 noet: */
